package com.brilliance.eibs.server;

import com.aliyun.openservices.ons.api.*;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;

import java.util.Properties;

public class RocketMQServerOns extends AbsServer {

    private Consumer consumer = null;

    public RocketMQServerOns(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "RocketMQOnsServer starting ." + LOG_FLAG);
        Properties properties;
        String CONSUMER_ID = "";        //消费者ID
        String ACCESS_KEY = "";        //阿里云身份验证AccessKey
        String SECRET_KEY = "";        //阿里云身份验证SecretKey
        String ONS_ADDR = "";        //消息队列服务器地址
        String NAMESRV_ADDR = "";        //消息队列服务器地址
        String[] TOPIC;    //消息主题组
        String[] TAG;        //消息标签组
        int CONSUME_THREAD_NUM = 0;  //消费线程数
        int MAX_RECONSUME_TIMES = 0;  //消息消费失败的最大重试次数
        int CONSUME_TIMEOUT = 0;  //每条消息消费的最大超时时间
        int SUSPEND_TIME_MILLIS = 0;  //消息消费失败的重试间隔时间(只适用于顺序消息)

        String interfaceName = getRequiredPropertyValue("interfaceName");
        String transactionName = getRequiredPropertyValue("transactionName");

        CONSUMER_ID = getRequiredPropertyValue("GroupId");
        ACCESS_KEY = getPropertyValue("AccessKey");
        SECRET_KEY = getPropertyValue("SecretKey");
        ONS_ADDR = getPropertyValue("ONSAddr");
        NAMESRV_ADDR = getPropertyValue("NamesrvAddr");
        TOPIC = getRequiredPropertyValue("topic").split(",");
        TAG = getRequiredPropertyValue("tag").split(",");
        CONSUME_THREAD_NUM = getPropertyValue("ConsumeThreadNums", 5);
        MAX_RECONSUME_TIMES = getPropertyValue("MaxReconsumeTimes", 1);
        CONSUME_TIMEOUT = getPropertyValue("ConsumeTimeout", 15);
        SUSPEND_TIME_MILLIS = getPropertyValue("SuspendTimeMillis", 3000);
        //校验topic和tag数量是否一致，支持多消息订阅
        checkTopicAndTag(TOPIC, TAG);
        properties = new Properties();
        //消息消费者ID
        properties.put(PropertyKeyConst.GROUP_ID, CONSUMER_ID);
        //阿里云身份验证AccessKey
        properties.put(PropertyKeyConst.AccessKey, ACCESS_KEY);
        //阿里云身份验证secretKey
        properties.put(PropertyKeyConst.SecretKey, SECRET_KEY);
        //消息队列服务器地址
        if (!StringUtil.isEmpty(ONS_ADDR)) {
            properties.put(PropertyKeyConst.ONSAddr, ONS_ADDR);
        } else if (!StringUtil.isEmpty(NAMESRV_ADDR)) {
            properties.put(PropertyKeyConst.NAMESRV_ADDR, NAMESRV_ADDR);
        } else {
            logger.error(LOG_FLAG + "不支持的接入方式." + LOG_FLAG);
            throw new IllegalAccessError("不支持的接入方式.");
        }
        //设置 Consumer 实例的消费线程数，默认：64 (订阅 消费者)
        properties.put(PropertyKeyConst.ConsumeThreadNums, CONSUME_THREAD_NUM);
        //设置消息消费失败的最大重试次数，默认：16 (订阅 消费者)
        properties.put(PropertyKeyConst.MaxReconsumeTimes, MAX_RECONSUME_TIMES);
        //设置每条消息消费的最大超时时间，超过设置时间则被视为消费失败， (订阅 消费者)
        //等下次重新投递再次消费。每个业务需要设置一个合理的值，单位（分钟）。默认：15
        properties.put(PropertyKeyConst.ConsumeTimeout, CONSUME_TIMEOUT);
        //只适用于顺序消息，设置消息消费失败的重试间隔时间 单位(毫秒) (订阅 消费者)
        properties.put(PropertyKeyConst.SuspendTimeMillis, SUSPEND_TIME_MILLIS);
        try {
            logger.info("开始创建Consumer...");
            consumer = ONSFactory.createConsumer(properties);
            for (int i = 0; i < TOPIC.length; i++) {
                String tp = TOPIC[i];
                String tg = TAG[i];
                if (StringUtil.isEmpty(tp))
                    continue;
                logger.info("订阅的Topic" + i + "：" + tp);
                logger.info("订阅的Tag" + i + "：" + tg);
                consumer.subscribe(tp, tg, new MessageListenerImpl(interfaceName, transactionName, context, logger));
            }
            consumer.start();
        } catch (Exception e) {
            logger.error(LOG_FLAG + "RocketMQOnsServer error." + LOG_FLAG, e);
            close();
        }

        logger.info(LOG_FLAG + "RocketMQOnsServer is finished." + LOG_FLAG);
    }

    private void checkTopicAndTag(String[] topic, String[] tag) {
        if (topic.length != tag.length) {
            throw new IllegalArgumentException("消息主题【TOPIC】和消息标签【TAG】数量不一致!消息主题有" + topic.length + "个，消息标签有" + tag.length);
        }
    }

    @Override
    public void close() {
        if (null != consumer)
            consumer.shutdown();
    }

}

class MessageListenerImpl implements MessageListener {
    private String interfaceName;
    private String transactionName;
    private Context context;
    private Logger logger;


    public MessageListenerImpl(String interfaceName, String transactionName, Context context, Logger logger) {
        super();
        this.interfaceName = interfaceName;
        this.transactionName = transactionName;
        this.context = context;
        this.logger = logger;
    }


    @Override
    public Action consume(Message message, ConsumeContext ctx) {
        String topicTagStr = "";
        String msgId = message.getMsgID();
        try {
            topicTagStr = message.getTopic() + "," + message.getTag();

            logger.info("msgId=" + msgId + "当前Topic+Tag=" + topicTagStr);
            // 查询消息消费处理
            Client client = new Client();
            client.call(interfaceName, transactionName, new Object[]{message.getBody(), msgId});
        } catch (Exception e) {
            logger.error("查询消息监听类异常:", e);
        }
        return Action.CommitMessage;
    }

}
